# NOTE: Derived from blib/lib/DBIx/YAWM.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package DBIx::YAWM;

#line 344 "blib/lib/DBIx/YAWM.pm (autosplit into blib/lib/auto/DBIx/YAWM/flatQuery.al)"
## flatQuery ######################################
## use DBI's built in fetchrow_hashref(), which
## (I'm guessing) is implemented in xs and far 
## superior to what we've been doing here for all
## these years. As such, we don't (necessarily)
## have to take an array ref on Select ... 
## hence the name 'flatQuery' ... we can take either
## a string or an array ... otherwise pretty much
## the same old.

sub flatQuery {

	my ($self, %p) = @_;
	
	#verify required inputs
	foreach ('Select', 'From'){
		exists($p{$_}) || do {
			$self->{'errstr'} = $_ . " is a required option to Query";
			return (undef);
		};
	}
	
	#select has to be an array reference or a string
	if ($#{$p{'Select'}} >= 0){
		
		#flatten select
		$p{'Select'} = join (', ', @{$p{'Select'}});
	
	}
	
	#error out if it's a null string
	(length($p{'Select'}) > 0) || do {
		$self->{'errstr'} = "'Select' option contains no fields to select!";
		return (undef);
	};
	
	#log into the database
	$self->Login() || do {
		$self->{'errstr'} = "Login failed: " . $self->{'errstr'};
		return (undef);
	};
	
	#struct query string
	my $QUERY = "select " . $p{'Options'} . " " . $p{'Select'} . " from " . $p{'From'};
	$QUERY   .= " where " . $p{'Where'} if ($p{'Where'} !~/^\s*$/);
	
	#prepare the query
	warn ("[Query]: preparing query ...") if ($self->{'Debug'} > 1);
	warn ("[Query]:\t $QUERY\n") if ($self->{'Debug'} > 1);
	my $sth = $self->{'dbh'}->prepare($QUERY) || do {
		$self->{errstr} = "Query: failed prepare: $QUERY / $DBI::errstr";
		return (undef);
	};
		
	#do it
	$sth->execute() || do {
	
	        #ok, right here ... new for 2.34
	        #we're going to detect ORA-ORA-03114 "not connected to oracle"
	        #we'll try to re-connect and re-execute.
	        if ( ($self->{'DBType'} eq "Oracle") && ($DBI::errstr =~/ORA-(\d{1,5})/i) ){
	                my $ora_code = $1;
	                
	                #if we wanted to detect other ora failure codes, here's the spot for it.
	                
	                #ORA-03114: try destroying the db handle and reconnecting
	                if ($ora_code eq "03114"){
	                        warn ("detected ORA-03114, attempting reconnect") if ($self->{'Debug'});
	                        $self->{dbh}->disconnect;
	                        delete($self->{dnh});
	                        $self->Login() || do {
	                                $self->{'errstr'}  = "[ORA-03114 encountered, reconnect failed] ";
	                                $self->{'errstr'} .= "can't execute query: " . $QUERY . " / " . $DBI::errstr;
	                                return(undef)
	                        };
	                        warn ("detected ORA-03114, reconnect succeeded, retrying query") if ($self->{'Debug'});
                                $sth->execute() || do {
                                        $self->{'errstr'}  = "[ORA-03114 encountered, reconnect success, but query still fails] ";
                                        $self->{'errstr'} = "can't execute query: " . $QUERY . " / " . $DBI::errstr;
                                        return (undef);
                                };
	                        
	                        
	                }
	        }else{
                        #just your run of the mill failure
                        $self->{'errstr'} = "can't execute query: " . $QUERY . " / " . $DBI::errstr;
                        return (undef);
                }
	
	};
	
	#well now dbi does hashref internal, and you know its got to be better than what we've been
	#up to ... so we're going to use this. with any luck, this'll handle the 'as' clause as well
	my (@out, $rec) = ();
	while ($rec = $sth->fetchrow_hashref()){ 
	
		#translate field names to lowercase if option supplied
		if ($p{'lc'} !~/^\s*$/){
			foreach my $k (keys %{$rec}){
				my $kl = lc($k);
				$rec->{$kl} = $rec->{$k};
				delete($rec->{$k}) if ($k ne $kl);
			}
		}
	
		push (@out, $rec); 
		
	}
	$sth->finish();
	warn ("retrieved: " . ($#out + 1) . " records") if $self->{'Debug'};
	
	#if no records returned
	($#out >= 0) || do {
		$self->{'errstr'} = "no records returned";
		return (undef);
	};
	
	#return what we got
	return (\@out);
}

1;
# end of DBIx::YAWM::flatQuery
